/*
**	BPALogin - lightweight portable BIDS2 login client
**	Copyright (c) 2001-3 Shane Hyde, and others.
** 
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
** 
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
** 
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
*/ 

#ifdef BIGPOND

#include "os.h"
#include "config.h"
#include "bpalogin.h"
#include "dns.h"

extern DNS_ServerEntry dns_server[];

/*
**  Main processing loop.  Logs in, handles heartbeats, and logs out on request
**
**  Returns - 0 - We are no longer connected, and should not retry.
**            1 - We are no longer connnected and should retry.
*/
int mainloop(struct session * s)
{
	int err;
	struct sockaddr_in listenaddr;
	struct hostent * he;
	int addrsize;
	char ip_str[25];

	s->lastheartbeat = 0;
	s->sequence = 0;
	s->listensock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);

	s->localaddr.sin_family = AF_INET;
	s->localaddr.sin_port = htons(DEFAULT_AUTHPORT1+1);  //htons(s->localport);
	s->localaddr.sin_addr.s_addr = INADDR_ANY;	

	addrsize = sizeof(struct sockaddr_in);
	err = bind(s->listensock,(struct sockaddr *)&s->localaddr,sizeof(s->localaddr));
	err = getsockname(s->listensock,(struct sockaddr *)&listenaddr,&addrsize);

	s->sessionid = NU_Retrieve_Clock()/100;

	s->listenport = ntohs(listenaddr.sin_port);
	strcpy(s->osname,"DNI-OS");
	strcpy(s->osrelease,"A1.00");

#if 0
GET_AUTH_SER:
    //get authserver IP from DNS server
    s->debug(0, "Get Authserver(%s) IP from DNS Server \r\n", s->authserver);  
    
#if 0
	he = gethostbyname(s->authserver);
    
	if(he)
	{
		s->authhost.sin_addr.s_addr = *((int*)(he->h_addr_list[0]));
	}
	else
	{
		s->authhost.sin_addr.s_addr = inet_addr(s->authserver);
	}
#else
    {
        int     connect_cnt;
        char    iobuf[1518], nameServer[40];
        int     have_ip = 0;
        unsigned int server_ip;
        
	    sprintf((char *)nameServer,"%d.%d.%d.%d",   (dns_server[LAN_NUM*MAX_DNS_SERVER].ip & 0xFF000000)>>24,
    	                                            (dns_server[LAN_NUM*MAX_DNS_SERVER].ip & 0x00FF0000)>>16,
        	                                        (dns_server[LAN_NUM*MAX_DNS_SERVER].ip & 0x0000FF00)>> 8,
            	                                    (dns_server[LAN_NUM*MAX_DNS_SERVER].ip & 0x000000FF)    );	                                                                                           
            	                                
    	for (connect_cnt=0 ; connect_cnt < 4 ; connect_cnt++)
    	{
    		have_ip = DNSResolver((BYTE *)s->authserver,(BYTE *)nameServer, 1, iobuf, 0); 
    		if (have_ip)
    			break;
    	}		
    	
    	if (!have_ip)
    	{
    	    /* can not resolve auth server ip, switch auth-server between dce-server and sm-server */
    	    s->debug(0,"can not resolve %s from DNS server(%s)\r\n", s->authserver, nameServer);
    	    bigpond_switch_server(&s);
    		goto GET_AUTH_SER;
    	}
    		   		 
    	memcpy((char *)&server_ip, iobuf, 4);
    	s->authhost.sin_addr.s_addr = server_ip;
    }		                                         
#endif	
    
    ifnIPAddrValueToString(s->authhost.sin_addr.s_addr, ip_str);
    s->debug(0, "Authserver IP -> %s", ip_str);

	s->authhost.sin_port = htons(s->authport);
	s->authhost.sin_family = AF_INET;

	if(config.isp.profile[0].telstra.authip)
	{
		ifnIPAddrValueToString(config.isp.profile[0].telstra.authip, ip_str);
		s->debug(0,"Auth host = %s:%d\n",ip_str,s->authport);
	}
	else		
		s->debug(0,"Auth host = %s:%d\n",s->authserver,s->authport);
#endif	
	s->debug(0,"Listening on port %d\n",s->listenport);

	if(login(s))
	{
	    s->debug(0, "logon() return 1");
		s->onconnected(s->listenport);
		if(!handle_heartbeats(s))
		{   //handle_heartbeats return 0, 
		    //Heartbeat timeout, and subsequent login failed to connect 
			int i;
			s->ondisconnected(1);
			s->noncritical("Sleeping for 5 seconds");
			closesocket(s->listensock);
			for(i=0;i<5;i++)
			{
				if(s->shutdown)
					return 0;
				else
					NU_Sleep(100);
			}
		}
		else
		{ //handle_heartbeats return 1, socket close by user
			closesocket(s->listensock);
			return 0;
		}
		
	}
	else
	{
		int i;
		s->debug(0, "logon() return 0");
		s->noncritical("login failed, Sleeping for 5 seconds");
		closesocket(s->listensock);
		for(i=0;i<5;i++)
		{
			if(s->shutdown)
				return 0;
			else
				NU_Sleep(100);
		}
	}
	return 1;
}

#endif
